@@ -27,8 +27,7 @@
  */
 
 /* 
- * MT safe ; FIXME: might still freeze, watch out, not thoroughly
- * looked at yet.  
+ * MT safe ; except for g_data*_foreach()
  */
 
 #include "config.h"
@@ -42,9 +41,14 @@
 
 /* --- defines --- */
 #define	G_QUARK_BLOCK_SIZE			(512)
-#define	G_DATA_MEM_CHUNK_PREALLOC		(128)
-#define	G_DATA_CACHE_MAX			(512)
-#define	G_DATASET_MEM_CHUNK_PREALLOC		(32)
+
+/* datalist pointer modifications have to be done with the g_dataset_global mutex held */
+#define G_DATALIST_GET_POINTER(datalist)						\
+  ((GData*) ((gsize) *(datalist) & ~(gsize) G_DATALIST_FLAGS_MASK))
+#define G_DATALIST_SET_POINTER(datalist, pointer) G_STMT_START {			\
+  *(datalist) = (GData*) (G_DATALIST_GET_FLAGS (datalist) |				\
+			  (gsize) pointer);						\
+} G_STMT_END
 
 
 /* --- structures --- */
@@ -82,11 +86,6 @@ G_LOCK_DEFINE_STATIC (g_dataset_global);
 static GHashTable   *g_dataset_location_ht = NULL;
 static GDataset     *g_dataset_cached = NULL; /* should this be
 						 threadspecific? */
-static GMemChunk    *g_dataset_mem_chunk = NULL;
-static GMemChunk    *g_data_mem_chunk = NULL;
-static GData	    *g_data_cache = NULL;
-static guint	     g_data_cache_length = 0;
-
 G_LOCK_DEFINE_STATIC (g_quark_global);
 static GHashTable   *g_quark_ht = NULL;
 static gchar       **g_quarks = NULL;
@@ -119,14 +118,7 @@ g_datalist_clear_i (GData **datalist)
 	  G_LOCK (g_dataset_global);
 	}
       
-      if (g_data_cache_length < G_DATA_CACHE_MAX)
-	{
-	  prev->next = g_data_cache;
-	  g_data_cache = prev;
-	  g_data_cache_length++;
-	}
-      else
-	g_mem_chunk_free (g_data_mem_chunk, prev);
+      g_slice_free (GData, prev);
     }
 }
 
@@ -174,7 +166,7 @@ g_dataset_destroy_internal (GDataset *dataset)
 	  if (dataset == g_dataset_cached)
 	    g_dataset_cached = NULL;
 	  g_hash_table_remove (g_dataset_location_ht, dataset_location);
-	  g_mem_chunk_free (g_dataset_mem_chunk, dataset);
+	  g_slice_free (GDataset, dataset);
 	  break;
 	}
       
@@ -229,7 +221,7 @@ g_data_set_internal (GData	  **datalist,
 		  G_DATALIST_SET_POINTER (datalist, list->next);
 		  
 		  /* the dataset destruction *must* be done
-		   * prior to invokation of the data destroy function
+		   * prior to invocation of the data destroy function
 		   */
 		  if (!list->next && dataset)
 		    g_dataset_destroy_internal (dataset);
@@ -250,14 +242,7 @@ g_data_set_internal (GData	  **datalist,
 	      else
 		ret_data = list->data;
 	      
-	      if (g_data_cache_length < G_DATA_CACHE_MAX)
-		{
-		  list->next = g_data_cache;
-		  g_data_cache = list;
-		  g_data_cache_length++;
-		}
-	      else
-		g_mem_chunk_free (g_data_mem_chunk, list);
+              g_slice_free (GData, list);
 	      
 	      return ret_data;
 	    }
@@ -288,7 +273,7 @@ g_data_set_internal (GData	  **datalist,
 		  list->destroy_func = destroy_func;
 		  
 		  /* we need to have updated all structures prior to
-		   * invokation of the destroy function
+		   * invocation of the destroy function
 		   */
 		  G_UNLOCK (g_dataset_global);
 		  dfunc (ddata);
@@ -301,14 +286,7 @@ g_data_set_internal (GData	  **datalist,
 	  list = list->next;
 	}
       
-      if (g_data_cache)
-	{
-	  list = g_data_cache;
-	  g_data_cache = list->next;
-	  g_data_cache_length--;
-	}
-      else
-	list = g_chunk_new (GData, g_data_mem_chunk);
+      list = g_slice_new (GData);
       list->next = G_DATALIST_GET_POINTER (datalist);
       list->id = key_id;
       list->data = data;
@@ -345,7 +323,7 @@ g_dataset_id_set_data_full (gconstpointer  dataset_location,
   dataset = g_dataset_lookup (dataset_location);
   if (!dataset)
     {
-      dataset = g_chunk_new (GDataset, g_dataset_mem_chunk);
+      dataset = g_slice_new (GDataset);
       dataset->location = dataset_location;
       g_datalist_init (&dataset->datalist);
       g_hash_table_insert (g_dataset_location_ht, 
@@ -453,18 +431,21 @@ gpointer
 g_datalist_id_get_data (GData	 **datalist,
 			GQuark     key_id)
 {
+  gpointer data = NULL;
   g_return_val_if_fail (datalist != NULL, NULL);
-  
   if (key_id)
     {
       register GData *list;
-      
+      G_LOCK (g_dataset_global);
       for (list = G_DATALIST_GET_POINTER (datalist); list; list = list->next)
 	if (list->id == key_id)
-	  return list->data;
+	  {
+            data = list->data;
+            break;
+          }
+      G_UNLOCK (g_dataset_global);
     }
-  
-  return NULL;
+  return data;
 }
 
 void
@@ -544,10 +525,16 @@ void
 g_datalist_set_flags (GData **datalist,
 		      guint   flags)
 {
+  gpointer oldvalue;
   g_return_if_fail (datalist != NULL);
   g_return_if_fail ((flags & ~G_DATALIST_FLAGS_MASK) == 0);
-
-  G_DATALIST_SET_FLAGS (datalist, flags);
+  
+  do
+    {
+      oldvalue = g_atomic_pointer_get (datalist);
+    }
+  while (!g_atomic_pointer_compare_and_exchange ((void**) datalist, oldvalue,
+                                                 (gpointer) ((gsize) oldvalue | flags)));
 }
 
 /**
@@ -565,10 +552,16 @@ void
 g_datalist_unset_flags (GData **datalist,
 			guint   flags)
 {
+  gpointer oldvalue;
   g_return_if_fail (datalist != NULL);
   g_return_if_fail ((flags & ~G_DATALIST_FLAGS_MASK) == 0);
-
-  G_DATALIST_UNSET_FLAGS (datalist, flags);
+  
+  do
+    {
+      oldvalue = g_atomic_pointer_get (datalist);
+    }
+  while (!g_atomic_pointer_compare_and_exchange ((void**) datalist, oldvalue,
+                                                 (gpointer) ((gsize) oldvalue & ~(gsize) flags)));
 }
 
 /**
@@ -584,8 +577,8 @@ guint
 g_datalist_get_flags (GData **datalist)
 {
   g_return_val_if_fail (datalist != NULL, 0);
-
-  return G_DATALIST_GET_FLAGS (datalist);
+  
+  return G_DATALIST_GET_FLAGS (datalist); /* atomic macro */
 }
 
 /* HOLDS: g_dataset_global_lock */
@@ -596,16 +589,6 @@ g_data_initialize (void)
 
   g_dataset_location_ht = g_hash_table_new (g_direct_hash, NULL);
   g_dataset_cached = NULL;
-  g_dataset_mem_chunk =
-    g_mem_chunk_new ("GDataset MemChunk",
-		     sizeof (GDataset),
-		     sizeof (GDataset) * G_DATASET_MEM_CHUNK_PREALLOC,
-		     G_ALLOC_AND_FREE);
-  g_data_mem_chunk =
-    g_mem_chunk_new ("GData MemChunk",
-		     sizeof (GData),
-		     sizeof (GData) * G_DATA_MEM_CHUNK_PREALLOC,
-		     G_ALLOC_AND_FREE);
 }
 
 GQuark
@@ -622,6 +605,22 @@ g_quark_try_string (const gchar *string)
   return quark;
 }
 
+/* HOLDS: g_quark_global_lock */
+static inline GQuark
+g_quark_from_string_internal (const gchar *string, 
+			      gboolean     duplicate)
+{
+  GQuark quark = 0;
+  
+  if (g_quark_ht)
+    quark = GPOINTER_TO_UINT (g_hash_table_lookup (g_quark_ht, string));
+  
+  if (!quark)
+    quark = g_quark_new (duplicate ? g_strdup (string) : (gchar *)string);
+  
+  return quark;
+}
+
 GQuark
 g_quark_from_string (const gchar *string)
 {
@@ -630,16 +629,7 @@ g_quark_from_string (const gchar *string)
   g_return_val_if_fail (string != NULL, 0);
   
   G_LOCK (g_quark_global);
-  if (g_quark_ht)
-    quark = (gulong) g_hash_table_lookup (g_quark_ht, string);
-  else
-    {
-      g_quark_ht = g_hash_table_new (g_str_hash, g_str_equal);
-      quark = 0;
-    }
-  
-  if (!quark)
-    quark = g_quark_new (g_strdup (string));
+  quark = g_quark_from_string_internal (string, TRUE);
   G_UNLOCK (g_quark_global);
   
   return quark;
@@ -653,18 +643,9 @@ g_quark_from_static_string (const gchar *string)
   g_return_val_if_fail (string != NULL, 0);
   
   G_LOCK (g_quark_global);
-  if (g_quark_ht)
-    quark = (gulong) g_hash_table_lookup (g_quark_ht, string);
-  else
-    {
-      g_quark_ht = g_hash_table_new (g_str_hash, g_str_equal);
-      quark = 0;
-    }
-
-  if (!quark)
-    quark = g_quark_new ((gchar*) string);
+  quark = g_quark_from_string_internal (string, FALSE);
   G_UNLOCK (g_quark_global);
- 
+
   return quark;
 }
 
@@ -672,9 +653,10 @@ G_CONST_RETURN gchar*
 g_quark_to_string (GQuark quark)
 {
   gchar* result = NULL;
+
   G_LOCK (g_quark_global);
-  if (quark > 0 && quark <= g_quark_seq_id)
-    result = g_quarks[quark - 1];
+  if (quark < g_quark_seq_id)
+    result = g_quarks[quark];
   G_UNLOCK (g_quark_global);
 
   return result;
@@ -688,14 +670,79 @@ g_quark_new (gchar *string)
   
   if (g_quark_seq_id % G_QUARK_BLOCK_SIZE == 0)
     g_quarks = g_renew (gchar*, g_quarks, g_quark_seq_id + G_QUARK_BLOCK_SIZE);
-  
-  g_quarks[g_quark_seq_id] = string;
-  g_quark_seq_id++;
-  quark = g_quark_seq_id;
+  if (!g_quark_ht)
+    {
+      g_assert (g_quark_seq_id == 0);
+      g_quark_ht = g_hash_table_new (g_str_hash, g_str_equal);
+      g_quarks[g_quark_seq_id++] = NULL;
+    }
+
+  quark = g_quark_seq_id++;
+  g_quarks[quark] = string;
   g_hash_table_insert (g_quark_ht, string, GUINT_TO_POINTER (quark));
   
   return quark;
 }
 
+/**
+ * g_intern_string:
+ * @string: a string
+ * 
+ * Returns a canonical representation for @string. Interned strings can
+ * be compared for equality by comparing the pointers, instead of using strcmp().
+ * 
+ * Returns: a canonical representation for the string
+ *
+ * Since: 2.10
+ */
+G_CONST_RETURN gchar*
+g_intern_string (const gchar *string)
+{
+  const gchar *result;
+  GQuark quark;
+
+  if (!string)
+    return NULL;
+
+  G_LOCK (g_quark_global);
+  quark = g_quark_from_string_internal (string, TRUE);
+  result = g_quarks[quark];
+  G_UNLOCK (g_quark_global);
+
+  return result;
+}
+
+/**
+ * g_intern_static_string:
+ * @string: a static string
+ * 
+ * Returns a canonical representation for @string. Interned strings can
+ * be compared for equality by comparing the pointers, instead of using strcmp().
+ * g_intern_static_string() does not copy the string, therefore @string must
+ * not be freed or modified. 
+ * 
+ * Returns: a canonical representation for the string
+ *
+ * Since: 2.10
+ */
+G_CONST_RETURN gchar*
+g_intern_static_string (const gchar *string)
+{
+  GQuark quark;
+  const gchar *result;
+
+  if (!string)
+    return NULL;
+
+  G_LOCK (g_quark_global);
+  quark = g_quark_from_string_internal (string, FALSE);
+  result = g_quarks[quark];
+  G_UNLOCK (g_quark_global);
+
+  return result;
+}
+
+
+
 #define __G_DATASET_C__
 #include "galiasdef.c"
