@@ -35,7 +35,6 @@
 /* #define DEBUG_MSG(args)	g_message args ; */
 #endif
 
-#include "galias.h"
 #include "glib.h"
 
 #include <time.h>
@@ -43,6 +42,8 @@
 #include <stdlib.h>
 #include <locale.h>
 
+#include "galias.h"
+
 GDate*
 g_date_new (void)
 {
@@ -251,7 +252,6 @@ g_date_update_dmy (const GDate *const_d)
 GDateWeekday 
 g_date_get_weekday (const GDate *d)
 {
-  g_return_val_if_fail (d != NULL, G_DATE_BAD_WEEKDAY);
   g_return_val_if_fail (g_date_valid (d), G_DATE_BAD_WEEKDAY);
   
   if (!d->julian) 
@@ -266,7 +266,6 @@ g_date_get_weekday (const GDate *d)
 GDateMonth   
 g_date_get_month (const GDate *d)
 {
-  g_return_val_if_fail (d != NULL, G_DATE_BAD_MONTH);
   g_return_val_if_fail (g_date_valid (d), G_DATE_BAD_MONTH);
   
   if (!d->dmy) 
@@ -281,7 +280,6 @@ g_date_get_month (const GDate *d)
 GDateYear    
 g_date_get_year (const GDate *d)
 {
-  g_return_val_if_fail (d != NULL, G_DATE_BAD_YEAR);
   g_return_val_if_fail (g_date_valid (d), G_DATE_BAD_YEAR);
   
   if (!d->dmy) 
@@ -296,7 +294,6 @@ g_date_get_year (const GDate *d)
 GDateDay     
 g_date_get_day (const GDate *d)
 {
-  g_return_val_if_fail (d != NULL, G_DATE_BAD_DAY);
   g_return_val_if_fail (g_date_valid (d), G_DATE_BAD_DAY);
   
   if (!d->dmy) 
@@ -311,7 +308,6 @@ g_date_get_day (const GDate *d)
 guint32      
 g_date_get_julian (const GDate *d)
 {
-  g_return_val_if_fail (d != NULL, G_DATE_BAD_JULIAN);
   g_return_val_if_fail (g_date_valid (d), G_DATE_BAD_JULIAN);
   
   if (!d->julian) 
@@ -328,7 +324,6 @@ g_date_get_day_of_year (const GDate *d)
 {
   gint index;
   
-  g_return_val_if_fail (d != NULL, 0);
   g_return_val_if_fail (g_date_valid (d), 0);
   
   if (!d->dmy) 
@@ -349,7 +344,6 @@ g_date_get_monday_week_of_year (const GDate *d)
   guint day;
   GDate first;
   
-  g_return_val_if_fail (d != NULL, 0);
   g_return_val_if_fail (g_date_valid (d), 0);
   
   if (!d->dmy) 
@@ -375,7 +369,6 @@ g_date_get_sunday_week_of_year (const GDate *d)
   guint day;
   GDate first;
   
-  g_return_val_if_fail (d != NULL, 0);
   g_return_val_if_fail (g_date_valid (d), 0);
   
   if (!d->dmy) 
@@ -411,7 +404,6 @@ g_date_get_iso8601_week_of_year (const GDate *d)
 {
   guint j, d4, L, d1, w;
 
-  g_return_val_if_fail (d != NULL, 0);
   g_return_val_if_fail (g_date_valid (d), 0);
   
   if (!d->julian)
@@ -422,7 +414,7 @@ g_date_get_iso8601_week_of_year (const GDate *d)
    * Julian Period which starts on 1 January 4713 BC, so we add
    * 1,721,425 to the number of days before doing the formula. 
    */
-  j  = d->julian + 1721425;
+  j  = d->julian_days + 1721425;
   d4 = (j + 31741 - (j % 7)) % 146097 % 36524 % 1461;
   L  = d4 / 1460;
   d1 = ((d4 - L) % 365) + L;
@@ -435,9 +427,6 @@ gint
 g_date_days_between (const GDate *d1,
 		     const GDate *d2)
 {
-  g_return_val_if_fail (d1 != NULL, 0);
-  g_return_val_if_fail (d2 != NULL, 0);
-
   g_return_val_if_fail (g_date_valid (d1), 0);
   g_return_val_if_fail (g_date_valid (d2), 0);
 
@@ -445,7 +434,7 @@ g_date_days_between (const GDate *d1,
 }
 
 void         
-g_date_clear (GDate       *d, guint ndates)
+g_date_clear (GDate *d, guint ndates)
 {
   g_return_if_fail (d != NULL);
   g_return_if_fail (ndates != 0);
@@ -484,7 +473,7 @@ static GDateDMY dmy_order[3] =
  * are counted as in the year 1900.  
  */
 
-static GDateYear twodigit_start_year = 1930;
+static const GDateYear twodigit_start_year = 1930;
 
 /* It is impossible to enter a year between 1 AD and 99 AD with this
  * in effect.  
@@ -870,20 +859,33 @@ g_date_set_parse (GDate       *d,
   G_UNLOCK (g_date_global);
 }
 
+/**
+ * g_date_set_time_t:
+ * @date: a #GDate 
+ * @timet: <type>time_t</type> value to set
+ *
+ * Sets the value of a date from a <type>time_t</type> value. 
+ *
+ * To set the value of a date to the current day, you could write:
+ * <informalexample><programlisting> 
+ *  g_date_set_time_t (date, time (NULL)); 
+ * </programlisting></informalexample>
+ *
+ * Since: 2.10
+ */
 void         
-g_date_set_time (GDate *d,
-		 GTime  time)
+g_date_set_time_t (GDate *date,
+		   time_t timet)
 {
-  time_t t = time;
   struct tm tm;
   
-  g_return_if_fail (d != NULL);
+  g_return_if_fail (date != NULL);
   
 #ifdef HAVE_LOCALTIME_R
-  localtime_r (&t, &tm);
+  localtime_r (&timet, &tm);
 #else
   {
-    struct tm *ptm = localtime (&t);
+    struct tm *ptm = localtime (&timet);
 
     if (ptm == NULL)
       {
@@ -903,15 +905,50 @@ g_date_set_time (GDate *d,
   }
 #endif
   
-  d->julian = FALSE;
+  date->julian = FALSE;
   
-  d->month = tm.tm_mon + 1;
-  d->day   = tm.tm_mday;
-  d->year  = tm.tm_year + 1900;
+  date->month = tm.tm_mon + 1;
+  date->day   = tm.tm_mday;
+  date->year  = tm.tm_year + 1900;
   
-  g_return_if_fail (g_date_valid_dmy (d->day, d->month, d->year));
+  g_return_if_fail (g_date_valid_dmy (date->day, date->month, date->year));
   
-  d->dmy    = TRUE;
+  date->dmy    = TRUE;
+}
+
+
+/**
+ * g_date_set_time:
+ * @date: a #GDate.
+ * @time_: #GTime value to set.
+ *
+ * Sets the value of a date from a #GTime value. 
+ *
+ * @Deprecated:2.10: Use g_date_set_time_t() instead.
+ */
+void
+g_date_set_time (GDate    *date,
+		 GTime    *time_)
+{
+  g_date_set_time_t (date, (time_t) time_);
+}
+
+/**
+ * g_date_set_time_val:
+ * @date: a #GDate 
+ * @timeval: #GTimeVal value to set
+ *
+ * Sets the value of a date from a #GTimeVal value.  Note that the
+ * @tv_usec member is ignored, because #GDate can't make use of the
+ * additional precision.
+ *
+ * Since: 2.10
+ */
+void
+g_date_set_time_val (GDate    *date,
+		     GTimeVal *timeval)
+{
+  g_date_set_time_t (date, (time_t) timeval->tv_sec);
 }
 
 void         
@@ -1001,7 +1038,6 @@ g_date_set_julian (GDate *d, guint32 j)
 gboolean     
 g_date_is_first_of_month (const GDate *d)
 {
-  g_return_val_if_fail (d != NULL, FALSE);
   g_return_val_if_fail (g_date_valid (d), FALSE);
   
   if (!d->dmy) 
@@ -1019,7 +1055,6 @@ g_date_is_last_of_month (const GDate *d)
 {
   gint index;
   
-  g_return_val_if_fail (d != NULL, FALSE);
   g_return_val_if_fail (g_date_valid (d), FALSE);
   
   if (!d->dmy) 
@@ -1037,7 +1072,6 @@ g_date_is_last_of_month (const GDate *d)
 void         
 g_date_add_days (GDate *d, guint ndays)
 {
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   
   if (!d->julian)
@@ -1053,7 +1087,6 @@ g_date_add_days (GDate *d, guint ndays)
 void         
 g_date_subtract_days (GDate *d, guint ndays)
 {
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   
   if (!d->julian)
@@ -1074,7 +1107,6 @@ g_date_add_months (GDate       *d,
   guint years, months;
   gint index;
   
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   
   if (!d->dmy) 
@@ -1108,7 +1140,6 @@ g_date_subtract_months (GDate       *d,
   guint years, months;
   gint index;
   
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   
   if (!d->dmy) 
@@ -1146,7 +1177,6 @@ void
 g_date_add_years (GDate       *d, 
                   guint        nyears)
 {
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   
   if (!d->dmy) 
@@ -1172,7 +1202,6 @@ void
 g_date_subtract_years (GDate       *d, 
                        guint        nyears)
 {
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   
   if (!d->dmy) 
@@ -1320,7 +1349,6 @@ g_date_to_struct_tm (const GDate *d,
 {
   GDateWeekday day;
      
-  g_return_if_fail (d != NULL);
   g_return_if_fail (g_date_valid (d));
   g_return_if_fail (tm != NULL);
   
@@ -1358,8 +1386,8 @@ g_date_clamp (GDate *date,
 	      const GDate *min_date,
 	      const GDate *max_date)
 {
-  g_return_if_fail (date);
   g_return_if_fail (g_date_valid (date));
+
   if (min_date != NULL)
     g_return_if_fail (g_date_valid (min_date));
   if (max_date != NULL)
@@ -1378,8 +1406,6 @@ void
 g_date_order (GDate *date1,
               GDate *date2)
 {
-  g_return_if_fail (date1 != NULL);
-  g_return_if_fail (date2 != NULL);
   g_return_if_fail (g_date_valid (date1));
   g_return_if_fail (g_date_valid (date2));
 
@@ -1408,7 +1434,6 @@ g_date_strftime (gchar       *s,
   GError *error = NULL;
   gsize retval;
 
-  g_return_val_if_fail (d != NULL, 0);
   g_return_val_if_fail (g_date_valid (d), 0);
   g_return_val_if_fail (slen > 0, 0); 
   g_return_val_if_fail (format != 0, 0);
@@ -1491,3 +1516,6 @@ g_date_strftime (gchar       *s,
   return retval;
 }
 
+#define __G_DATE_C__
+#include "galiasdef.c"
+
