@@ -62,11 +62,11 @@ typedef void (*GWeakNotify)		(gpointer      data,
 					 GObject      *where_the_object_was);
 struct  _GObject
 {
-  GTypeInstance g_type_instance;
+  GTypeInstance  g_type_instance;
   
   /*< private >*/
-  guint         ref_count;
-  GData        *qdata;
+  volatile guint ref_count;
+  GData         *qdata;
 };
 struct  _GObjectClass
 {
@@ -140,16 +140,16 @@ GObject*    g_object_new_valist               (GType           object_type,
 					       va_list         var_args);
 void	    g_object_set                      (gpointer	       object,
 					       const gchar    *first_property_name,
-					       ...);
+					       ...) G_GNUC_NULL_TERMINATED;
 void        g_object_get                      (gpointer        object,
 					       const gchar    *first_property_name,
-					       ...);
+					       ...) G_GNUC_NULL_TERMINATED;
 gpointer    g_object_connect                  (gpointer	       object,
 					       const gchar    *signal_spec,
-					       ...);
+					       ...) G_GNUC_NULL_TERMINATED;
 void	    g_object_disconnect               (gpointer	       object,
 					       const gchar    *signal_spec,
-					       ...);
+					       ...) G_GNUC_NULL_TERMINATED;
 void        g_object_set_valist               (GObject        *object,
 					       const gchar    *first_property_name,
 					       va_list         var_args);
@@ -166,6 +166,8 @@ void        g_object_freeze_notify            (GObject        *object);
 void        g_object_notify                   (GObject        *object,
 					       const gchar    *property_name);
 void        g_object_thaw_notify              (GObject        *object);
+gboolean    g_object_is_floating    	      (gpointer        object);
+gpointer    g_object_ref_sink       	      (gpointer	       object);
 gpointer    g_object_ref                      (gpointer        object);
 void        g_object_unref                    (gpointer        object);
 void	    g_object_weak_ref		      (GObject	      *object,
@@ -178,6 +180,18 @@ void        g_object_add_weak_pointer         (GObject        *object,
                                                gpointer       *weak_pointer_location);
 void        g_object_remove_weak_pointer      (GObject        *object, 
                                                gpointer       *weak_pointer_location);
+
+typedef void (*GToggleNotify) (gpointer      data,
+			       GObject      *object,
+			       gboolean      is_last_ref);
+
+void g_object_add_toggle_ref    (GObject       *object,
+				 GToggleNotify  notify,
+				 gpointer       data);
+void g_object_remove_toggle_ref (GObject       *object,
+				 GToggleNotify  notify,
+				 gpointer       data);
+
 gpointer    g_object_get_qdata                (GObject        *object,
 					       GQuark          quark);
 void        g_object_set_qdata                (GObject        *object,
@@ -220,6 +234,7 @@ gulong	    g_signal_connect_object           (gpointer	       instance,
 
 
 /*< protected >*/
+void        g_object_force_floating           (GObject        *object);
 void        g_object_run_dispose	      (GObject	      *object);
 
 
